#ifndef F_CPU
#define F_CPU 4000000UL
#endif

#include <avr/io.h>
#include <util/delay.h>
#include "morse.h"

const uint8_t code[92] = { // not all official morse characters are implemented
	0,			// space	
	0,			// !
	0,			// "
	0,			// #
	0,			// $
	0,			// %
	0,			// &
	0,			// '
	0,			// (
	0,			// )
	0,			// *
	0,			// +
	0,			// ,
	0,			// -
	0,			// .
	0,			// /
	0b00111111,	// 0
	0b00111110,	// 1
	0b00111100,	// 2
	0b00111000,	// 3
	0b00110000,	// 4
	0b00100000,	// 5
	0b00100001,	// 6
	0b00100011,	// 7
	0b00100111,	// 8
	0b00101111,	// 9	// ASCII 
	0,
	0,
	0,
	0,			// =
	0,
	0,			// ?
	0,			// @
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0b00000110,	// a
	0b00010001,	// b
	0b00010101, // ...
	0b00001001,
	0b00000010,
	0b00010100,
	0b00001011,
	0b00010000,
	0b00000100,
	0b00011110,
	0b00001101,
	0b00010010,
	0b00000111,
	0b00000101,
	0b00001111,
	0b00010110,
	0b00011011,
	0b00001010,
	0b00001000,
	0b00000011,
	0b00001100,
	0b00011000,
	0b00001110,
	0b00011001,
	0b00011101,	// ...
	0b00010011,	// z
};

void morse_char( uint8_t ascii_char )
{
	ascii_char -= 32;
	if( ascii_char == 0 ) // space
	{
		_delay_ms(WORD_LENGTH);
	}
	else
	{		
		uint8_t work_byte;
		uint8_t morse_byte = code[ascii_char];
		
		do
		{
			work_byte = 0b00000001 & morse_byte;	// get the LSB
			PORTB |= (1<<PB0);
			SOUND_ON();
			
			if( work_byte == 1 )	// if LSB is 1 it's a DAH
				_delay_ms(DAH_LENGTH);
			else if( work_byte == 0 )	// else it's a DIT
				_delay_ms(DIT_LENGTH);
				
			PORTB &= !(1<<PB0);
			SOUND_OFF();
			_delay_ms(DIT_LENGTH);	// pause between symbols
			morse_byte = morse_byte>>1;		// shift the morse_byte one position to the right
		}
		while( morse_byte != 1 );	// if the Value ofthe remaining byte is 1, the character is done
	}
}

void morse_string( const char* string )
{
	while( *string ) {
		morse_char( *string );
		string++;
		_delay_ms(DAH_LENGTH);	// pause of dah between two characters of one word
	}
}
